use serial::{unix::TTYPort, BaudRate::BaudOther, SerialPort};
use std::{
    io::{Read, Write},
    thread::sleep,
    time::Duration,
};

use crate::packet::FecPacket;

const BUFFER_STATUS_CMD: u8 = 0x00;
const SEND_PACKET_CMD: u8 = 0x01;
const GET_TEMP_CMD: u8 = 0x02;

// Used for talking to an RF4463 via a microcontroller
// The microcontroller takes in packet data over UART and sends it to the RF4463
pub struct UartRadio {
    uart: TTYPort,
}

impl UartRadio {
    pub fn new(uart: &str) -> anyhow::Result<Self> {
        let mut uart = serial::open(uart)?;
        uart.reconfigure(&|settings| settings.set_baud_rate(BaudOther(921_600)))?;

        Ok(Self { uart })
    }

    pub fn send_packet(&mut self, packet: &FecPacket) -> anyhow::Result<()> {
        self.uart.write_all(&[SEND_PACKET_CMD])?;
        self.uart.write_all(&packet.0)?;

        let mut buf = [0; 1];
        self.uart.read_exact(&mut buf)?;

        // wait until we have space for packets
        while buf != [0x02] {
            sleep(Duration::from_millis(10));

            // poll to see if the buffer has emptied yet
            self.uart.write_all(&[BUFFER_STATUS_CMD])?;
            self.uart.flush()?;

            self.uart.read_exact(&mut buf)?;
        }

        Ok(())
    }

    pub fn get_temp(&mut self) -> anyhow::Result<f32> {
        self.uart.write_all(&[GET_TEMP_CMD])?;

        let mut buf = [0; 4];
        self.uart.read_exact(&mut buf)?;

        let temp = f32::from_le_bytes(buf);

        Ok(temp)
    }
}
